==> 1.7.2.tar.gz: test/view_models/widget_capture_viewmodel_test.dart <==
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:testsweets/src/enums/capture_widget_enum.dart';
import 'package:testsweets/src/enums/popup_menu_action.dart';
import 'package:testsweets/src/enums/toast_type.dart';
import 'package:testsweets/src/enums/widget_type.dart';
import 'package:testsweets/src/models/application_models.dart';
import 'package:testsweets/src/ui/widget_capture/widget_capture_view.form.dart';
import 'package:testsweets/src/ui/widget_capture/widget_capture_viewmodel.dart';

import '../helpers/test_consts.dart';
import '../helpers/test_helpers.dart';

final _projectId = 'projectId';
WidgetCaptureViewModel _getViewModel() {
  var model = WidgetCaptureViewModel(projectId: _projectId);
  model.screenCenterPosition = WidgetPosition.empty();
  return model;
}

void main() {
  group('WidgetCaptureViewModelTest -', () {
    setUp(registerServices);
    tearDown(unregisterServices);
    group('constructor -', () {
      test('When called, should set the project id in WidgetCaptureService',
          () async {
        final service = getAndRegisterWidgetCaptureService();

        _getViewModel();
        verify(service.projectId = _projectId);
      });
    });
    group('saveWidget -', () {
      test('''When called on route `current route`,
           Should added it to widgetDescription and send it to backend''',
          () async {
        getAndRegisterTestSweetsRouteTracker(currentRoute: 'current route');
        final model = _getViewModel();
        model.formValueMap[WidgetNameValueKey] = 'myWidgetName';
        model.showWidgetForm();
        model.widgetDescription!.copyWith(widgetType: WidgetType.scrollable);
        model.saveWidget();
        expect(model.widgetDescription!.originalViewName, 'current route');
      });
      test('''When called and the current route is `/current route`,
           Should convert it to `currentRoute` in viewName proberty before send it to backend''',
          () async {
        getAndRegisterTestSweetsRouteTracker(currentRoute: '/current route');
        final model = _getViewModel();
        model.formValueMap[WidgetNameValueKey] = 'myWidgetName';
        model.showWidgetForm();
        model.widgetDescription!.copyWith(widgetType: WidgetType.scrollable);
        model.saveWidget();
        expect(model.widgetDescription!.viewName, 'currentRoute');
        expect(model.widgetDescription!.originalViewName, '/current route');
      });
      test('''When called and the current widget name is `login-button`,
           Should convert it to `loginButton` in name proberty before send it to backend''',
          () async {
        getAndRegisterTestSweetsRouteTracker(currentRoute: '/current route');
        final model = _getViewModel();
        model.showWidgetForm();

        model.formValueMap[WidgetNameValueKey] = 'login-button';

        model.widgetDescription =
            model.widgetDescription!.copyWith(widgetType: WidgetType.touchable);
        model.saveWidget();
        expect(model.widgetDescription!.name, 'loginButton');
      });
    });

    group('updateWidgetDescription -', () {
      test(
          'When change the name, should call updateWidgetDescription() in WidgetCaptureService with the new name"',
          () async {
        final service = getAndRegisterWidgetCaptureService();

        final model = _getViewModel();
        await model.popupMenuActionSelected(
            kWidgetDescription, PopupMenuAction.edit);
        model.formValueMap[WidgetNameValueKey] = 'loginButton';
        model.setFormStatus();

        await model.updateWidgetDescription();

        verify(service.updateWidgetDescription(
            description: kWidgetDescription.copyWith(name: 'loginButton')));
      });

      test(
          'When called and update was successful, Should set the current CaptureWidgetStatusEnum to inspectMode',
          () async {
        final model = _getViewModel();

        model.showWidgetForm();

        model.formValueMap[WidgetNameValueKey] = 'loginButton';

        await model.updateWidgetDescription();

        expect(model.captureWidgetStatusEnum, CaptureWidgetStatusEnum.idle);
      });
    });

    group('removeWidgetDescription -', () {
      test(
          'When called, Should call deleteWidgetDescription from WidgetCaptureService',
          () async {
        final widgetCaptureService = getAndRegisterWidgetCaptureService();
        final model = _getViewModel();
        model.popupMenuActionSelected(
            kWidgetDescription, PopupMenuAction.remove);

        await model.removeWidgetDescription();
        verify(widgetCaptureService.deleteWidgetDescription(
            description: kWidgetDescription));
      });

      test(
          'When the call ended sucessfully, Should reset the captureWidgetStatusEnum to idle state',
          () async {
        final model = _getViewModel();
        model.showWidgetForm();
        await model.removeWidgetDescription();
        expect(model.captureWidgetStatusEnum, CaptureWidgetStatusEnum.idle);
      });
    });

    group('popupMenuActionSelected -', () {
      test(
          'When popupMenuAction is edit, Should set the captureWidgetStatusEnum to editWidget',
          () {
        final model = _getViewModel();
        model.popupMenuActionSelected(kWidgetDescription, PopupMenuAction.edit);
        expect(
            model.captureWidgetStatusEnum, CaptureWidgetStatusEnum.editWidget);
      });
      test(
          'When popupMenuAction is remove, Should call deleteWidgetDescription from captureService',
          () {
        final widgetCaptureService = getAndRegisterWidgetCaptureService();

        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription, PopupMenuAction.remove);
        verify(widgetCaptureService.deleteWidgetDescription(
            description: kWidgetDescription));
      });
      test(
          'When popupMenuAction is attachToKey, Should call deleteWidgetDescription from captureService',
          () {
        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription, PopupMenuAction.attachToKey);
        expect(model.captureWidgetStatusEnum,
            CaptureWidgetStatusEnum.attachWidget);
      });
      test('''When popupMenuAction is attachToKey, Should show
           a toast to let the user know that he need to choose a target''', () {
        final snackbarService = getAndRegisterSnackbarService();
        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription, PopupMenuAction.attachToKey);
        verify(snackbarService.showCustomSnackBar(
            message: 'Select Key to associate with Scroll View',
            variant: SnackbarType.info));
      });
    });
    group('addNewTarget -', () {
      test(
          '''When selecting target widget but before calling updateWidgetDescription,
           Should add it to current widgetDescription target ids list''',
          () async {
        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription, PopupMenuAction.attachToKey);

        /// I didn't add await inorder to expect the state before the updateWidgetDescription call
        model.addNewTarget('targetId');
        expect(model.widgetDescription!.targetIds.first, 'targetId');
        expect(model.captureWidgetStatusEnum,
            CaptureWidgetStatusEnum.attachWidget);
      });

      test(
          '''After adding a new target widget and update the widget on the backend,
           Should set the widgetDescription to null and captureWidgetStatusEnum to idle''',
          () async {
        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription, PopupMenuAction.attachToKey);

        await model.addNewTarget('targetId');
        expect(model.widgetDescription, isNull);
        expect(model.captureWidgetStatusEnum, CaptureWidgetStatusEnum.idle);
      });
    });

    group('removeTarget -', () {
      test('''When called and target is exist in targetIds list,
       Should remove the targetId from the list of the source/s widget''',
          () async {
        final model = _getViewModel();

        await model.popupMenuActionSelected(
            kWidgetDescription.copyWith(targetIds: ['targetId']),
            PopupMenuAction.deattachFromKey);
        model.removeTarget('targetId');
        expect(model.widgetDescription!.targetIds, isEmpty);
      });
      test('''When called and target doesn't exist in targetIds list,
       Should return 'Target doesn\'t exist' ''', () async {
        final model = _getViewModel();

        await model.popupMenuActionSelected(
            kWidgetDescription.copyWith(targetIds: []),
            PopupMenuAction.deattachFromKey);
        model.removeTarget('targetId');
        expect(await model.removeTarget('targetId'), 'Target doesn\'t exist');
      });
    });
  });
}

==> 1.7.5.tar.gz: test/view_models/widget_capture_viewmodel_test.dart <==
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:testsweets/src/enums/capture_widget_enum.dart';
import 'package:testsweets/src/enums/popup_menu_action.dart';
import 'package:testsweets/src/enums/toast_type.dart';
import 'package:testsweets/src/enums/widget_type.dart';
import 'package:testsweets/src/models/application_models.dart';
import 'package:testsweets/src/ui/widget_capture/widget_capture_view_form.dart';
import 'package:testsweets/src/ui/widget_capture/widget_capture_viewmodel.dart';

import '../helpers/test_consts.dart';
import '../helpers/test_helpers.dart';

final _projectId = 'projectId';
WidgetCaptureViewModel _getViewModel() {
  var model = WidgetCaptureViewModel(projectId: _projectId);
  model.screenCenterPosition = WidgetPosition.empty();
  return model;
}

void main() {
  group('WidgetCaptureViewModelTest -', () {
    setUp(registerServices);
    tearDown(unregisterServices);
    group('constructor -', () {
      test('When called, should set the project id in WidgetCaptureService',
          () async {
        final service = getAndRegisterWidgetCaptureService();

        _getViewModel();
        verify(service.projectId = _projectId);
      });
    });
    group('saveWidget -', () {
      test('''When called on route `current route`,
           Should added it to widgetDescription and send it to backend''',
          () async {
        getAndRegisterTestSweetsRouteTracker(currentRoute: 'current route');
        final model = _getViewModel();
        model.formValueMap[WidgetNameValueKey] = 'myWidgetName';
        model.showWidgetForm();
        model.widgetDescription!.copyWith(widgetType: WidgetType.scrollable);
        model.saveWidget();
        expect(model.widgetDescription!.originalViewName, 'current route');
      });
      test('''When called and the current route is `/current route`,
           Should convert it to `currentRoute` in viewName proberty before send it to backend''',
          () async {
        getAndRegisterTestSweetsRouteTracker(currentRoute: '/current route');
        final model = _getViewModel();
        model.formValueMap[WidgetNameValueKey] = 'myWidgetName';
        model.showWidgetForm();
        model.widgetDescription!.copyWith(widgetType: WidgetType.scrollable);
        model.saveWidget();
        expect(model.widgetDescription!.viewName, 'currentRoute');
        expect(model.widgetDescription!.originalViewName, '/current route');
      });
      test('''When called and the current widget name is `login-button`,
           Should convert it to `loginButton` in name proberty before send it to backend''',
          () async {
        getAndRegisterTestSweetsRouteTracker(currentRoute: '/current route');
        final model = _getViewModel();
        model.showWidgetForm();

        model.formValueMap[WidgetNameValueKey] = 'login-button';

        model.widgetDescription =
            model.widgetDescription!.copyWith(widgetType: WidgetType.touchable);
        model.saveWidget();
        expect(model.widgetDescription!.name, 'loginButton');
      });
    });

    group('updateWidgetDescription -', () {
      test(
          'When change the name, should call updateWidgetDescription() in WidgetCaptureService with the new name"',
          () async {
        final service = getAndRegisterWidgetCaptureService();

        final model = _getViewModel();
        await model.popupMenuActionSelected(
            kWidgetDescription2, PopupMenuAction.edit);
        model.formValueMap[WidgetNameValueKey] = 'loginButton';
        model.setFormStatus();

        await model.updateWidgetDescription();

        verify(service.updateWidgetDescription(
            description: kWidgetDescription2.copyWith(name: 'loginButton')));
      });

      test(
          'When called and update was successful, Should set the current CaptureWidgetStatusEnum to inspectMode',
          () async {
        final model = _getViewModel();

        model.showWidgetForm();

        model.formValueMap[WidgetNameValueKey] = 'loginButton';

        await model.updateWidgetDescription();

        expect(model.captureWidgetStatusEnum, CaptureWidgetStatusEnum.idle);
      });
    });

    group('removeWidgetDescription -', () {
      test(
          'When called, Should call deleteWidgetDescription from WidgetCaptureService',
          () async {
        final widgetCaptureService = getAndRegisterWidgetCaptureService();
        final model = _getViewModel();
        model.popupMenuActionSelected(
            kWidgetDescription2, PopupMenuAction.remove);

        await model.removeWidgetDescription();
        verify(widgetCaptureService.deleteWidgetDescription(
            description: kWidgetDescription2));
      });

      test(
          'When the call ended sucessfully, Should reset the captureWidgetStatusEnum to idle state',
          () async {
        final model = _getViewModel();
        model.showWidgetForm();
        await model.removeWidgetDescription();
        expect(model.captureWidgetStatusEnum, CaptureWidgetStatusEnum.idle);
      });
    });

    group('popupMenuActionSelected -', () {
      test(
          'When popupMenuAction is edit, Should set the captureWidgetStatusEnum to editWidget',
          () {
        final model = _getViewModel();
        model.popupMenuActionSelected(
            kWidgetDescription2, PopupMenuAction.edit);
        expect(
            model.captureWidgetStatusEnum, CaptureWidgetStatusEnum.editWidget);
      });
      test(
          'When popupMenuAction is remove, Should call deleteWidgetDescription from captureService',
          () {
        final widgetCaptureService = getAndRegisterWidgetCaptureService();

        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription2, PopupMenuAction.remove);
        verify(widgetCaptureService.deleteWidgetDescription(
            description: kWidgetDescription2));
      });
      test(
          'When popupMenuAction is attachToKey, Should call deleteWidgetDescription from captureService',
          () {
        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription2, PopupMenuAction.attachToKey);
        expect(model.captureWidgetStatusEnum,
            CaptureWidgetStatusEnum.attachWidget);
      });
      test('''When popupMenuAction is attachToKey, Should show
           a toast to let the user know that he need to choose a target''', () {
        final snackbarService = getAndRegisterSnackbarService();
        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription2, PopupMenuAction.attachToKey);
        verify(snackbarService.showCustomSnackBar(
            message: 'Select Key to associate with Scroll View',
            variant: SnackbarType.info));
      });
    });
    group('addNewTarget -', () {
      test(
          '''When selecting target widget but before calling updateWidgetDescription,
           Should add it to current widgetDescription target ids list''',
          () async {
        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription2, PopupMenuAction.attachToKey);

        /// I didn't add await inorder to expect the state before the updateWidgetDescription call
        model.addNewTarget('targetId');
        expect(model.widgetDescription!.targetIds.first, 'targetId');
        expect(model.captureWidgetStatusEnum,
            CaptureWidgetStatusEnum.attachWidget);
      });

      test(
          '''After adding a new target widget and update the widget on the backend,
           Should set the widgetDescription to null and captureWidgetStatusEnum to idle''',
          () async {
        final model = _getViewModel();

        model.popupMenuActionSelected(
            kWidgetDescription2, PopupMenuAction.attachToKey);

        await model.addNewTarget('targetId');
        expect(model.widgetDescription, isNull);
        expect(model.captureWidgetStatusEnum, CaptureWidgetStatusEnum.idle);
      });
    });

    group('removeTarget -', () {
      test('''When called and target is exist in targetIds list,
       Should remove the targetId from the list of the source/s widget''',
          () async {
        final model = _getViewModel();

        await model.popupMenuActionSelected(
            kWidgetDescription2.copyWith(targetIds: ['targetId']),
            PopupMenuAction.deattachFromKey);
        model.removeTarget('targetId');
        expect(model.widgetDescription!.targetIds, isEmpty);
      });
      test('''When called and target doesn't exist in targetIds list,
       Should return 'Target doesn\'t exist' ''', () async {
        final model = _getViewModel();

        await model.popupMenuActionSelected(
            kWidgetDescription2.copyWith(targetIds: []),
            PopupMenuAction.deattachFromKey);
        model.removeTarget('targetId');
        expect(await model.removeTarget('targetId'), 'Target doesn\'t exist');
      });
    });
  });
}
